/* uimanager.vala
 *
 * Copyright (C) 2008-2010 Nicolas Joseph
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Author:
 * 	Nicolas Joseph <nicolas.joseph@valaide.org>
 */

/**
 * Class for construct the user interface (menu and toolbar)
 */
public class Valide.UIManager : Gtk.UIManager
{
  private Gtk.ActionGroup action_group;

  construct
  {
    try
    {
      this.add_ui_from_string (this.xmldef, -1);
      this.action_group = new Gtk.ActionGroup ("general");
    }
    catch (Error e)
    {
      critical (e.message);
    }
  }

  /**
   * Populate the graphical user interface with menu and toolbar
   */
  public void populate ()
  {
    /* File menu */
    action_group.add_action (new Gtk.Action ("file", _("_File"), null, null));
    action_group.add_action_with_accel (new Gtk.Action ("file-new", null, _("Create new source file"), Gtk.STOCK_NEW), null);
    action_group.add_action_with_accel (new Gtk.Action ("file-open", _("Open..."), _("Open source file"), Gtk.STOCK_OPEN), null);
    action_group.add_action_with_accel (new Gtk.Action ("file-save", null, _("Save source file"), Gtk.STOCK_SAVE), null);
    action_group.add_action_with_accel (new Gtk.Action ("file-save-as", ("Save As..."), _("Save as source file"), Gtk.STOCK_SAVE_AS), "<ctrl><shift>S");
    action_group.add_action_with_accel (new Gtk.Action ("file-rename", ("Rename..."), _("Rename source file"), null), null);
    action_group.add_action_with_accel (new Gtk.Action ("file-close", null, _("Close source file"), Gtk.STOCK_CLOSE), null);
    action_group.add_action_with_accel (new Gtk.Action ("quit", null, _("Quit application"), Gtk.STOCK_QUIT), null);

    /* Edit menu */
    action_group.add_action (new Gtk.Action ("edit", _("_Edit"), null, null));
    action_group.add_action_with_accel (new Gtk.Action ("edit-undo", null, _("Undo last action"), Gtk.STOCK_UNDO), "<ctrl>Z");
    action_group.add_action_with_accel (new Gtk.Action ("edit-redo", null, _("Redo last undo action"), Gtk.STOCK_REDO), "<ctrl><shift>Z");
    action_group.add_action_with_accel (new Gtk.Action ("edit-copy", null, _("Copy text"), Gtk.STOCK_COPY), null);
    action_group.add_action_with_accel (new Gtk.Action ("edit-cut", null, _("Cut text"), Gtk.STOCK_CUT), null);
    action_group.add_action_with_accel (new Gtk.Action ("edit-paste", null, _("Paste text"), Gtk.STOCK_PASTE), null);
    action_group.add_action_with_accel (new Gtk.Action ("edit-lower", _("Make Lowercase"), _("Make selection lowercase"), null), "<ctrl>L");
    action_group.add_action_with_accel (new Gtk.Action ("edit-upper", _("Make Uppercase"), _("Make selection uppercase"), null), "<ctrl>U");
    action_group.add_action_with_accel (new Gtk.Action ("edit-preferences", null, _("Configure IDE"), Gtk.STOCK_PREFERENCES), null);

    /* Search menu */
    action_group.add_action (new Gtk.Action ("search", _("_Search"), null, null));
    action_group.add_action_with_accel (new Gtk.Action ("search-find", _("Find..."), _("Find"), Gtk.STOCK_FIND), null);
    action_group.add_action_with_accel (new Gtk.Action ("search-find-next", _("Find Next"), _("Find next"), null), "<ctrl>G");
    action_group.add_action_with_accel (new Gtk.Action ("search-find-prev", _("Find Previous"), _("Find previous"), null), "<ctrl><shift>G");
    action_group.add_action_with_accel (new Gtk.Action ("search-replace", null, _("Replace"), Gtk.STOCK_FIND_AND_REPLACE), "<ctrl>H");
    action_group.add_action_with_accel (new Gtk.Action ("search-goto-line", _("Go to Line..."), _("Goto line"), Gtk.STOCK_JUMP_TO), "<ctrl>I");

    /* View menu */
    action_group.add_action (new Gtk.Action ("view", _("_View"), null, null));
    action_group.add_action_with_accel (new Gtk.ToggleAction ("view-toolbar", _("Toolbar"), _("View/hide the toolbar"), null), null);
    action_group.add_action_with_accel (new Gtk.ToggleAction ("view-statusbar", _("Statusbar"), _("View/hide the statusbar"), null), null);
    action_group.add_action_with_accel (new Gtk.ToggleAction ("view-fullscreen", _("Full Screen"), _("View/hide the bottom panel"), null), "F11");

    /* Project menu */
    action_group.add_action (new Gtk.Action ("project", _("_Project"), null, null));
    action_group.add_action_with_accel (new Gtk.Action ("project-new", _("New..."), _("Create new project"), Gtk.STOCK_NEW), "<ctrl><shift>N");
    action_group.add_action_with_accel (new Gtk.Action ("project-open", _("Open..."), _("Open project"), Gtk.STOCK_OPEN), "<ctrl><shift>O");
    action_group.add_action_with_accel (new Gtk.Action ("project-add", _("_Add File..."), _("Add file to project"), Gtk.STOCK_ADD), null);
    action_group.add_action_with_accel (new Gtk.Action ("project-remove", _("_Remove File..."), _("Remove file to project"), Gtk.STOCK_REMOVE), null);
    action_group.add_action_with_accel (new Gtk.Action ("project-options", _("O_ptions..."), _("Project options"), Gtk.STOCK_PROPERTIES), null);
    action_group.add_action_with_accel (new Gtk.Action ("project-configure", _("_Configure"), _("Configure project"), null), "F5");
    action_group.add_action_with_accel (new Gtk.Action ("project-build", _("_Build"), _("Build project"), Gtk.STOCK_CONVERT), "F6");
    action_group.add_action_with_accel (new Gtk.Action ("project-install", _("_Install"), _("Install project"), null), "F7");
    action_group.add_action_with_accel (new Gtk.Action ("project-dist", _("_Dist"), _("Distribute project"), null), null);
    action_group.add_action_with_accel (new Gtk.Action ("project-clean", _("_Clean"), _("Clean project"), Gtk.STOCK_CLEAR), null);
    action_group.add_action_with_accel (new Gtk.Action ("project-distclean", _("_Distclean"), _("Clean project for distribution"), null), null);
    action_group.add_action_with_accel (new Gtk.Action ("project-uninstall", _("_Uninstall"), _("Uninstall project"), null), null);
    action_group.add_action_with_accel (new Gtk.Action ("project-execute", null, _("Execute project"), Gtk.STOCK_EXECUTE), "F8");
    action_group.add_action_with_accel (new Gtk.Action ("project-abort", _("Abort"), _("Abort executing"), Gtk.STOCK_STOP), "<shift>F8");
    action_group.add_action_with_accel (new Gtk.Action ("project-close", null, _("Close project"), Gtk.STOCK_CLOSE), "");

    /* Document menu */
    action_group.add_action (new Gtk.Action ("document", _("_Documents"), null, null));
    action_group.add_action_with_accel (new Gtk.Action ("document-save-all", ("Save All"), _("Save all unsaved source files"), Gtk.STOCK_SAVE), "<ctrl><shift>L");
    action_group.add_action_with_accel (new Gtk.Action ("document-close-all", _("C_lose All"), _("Close all source files"), Gtk.STOCK_CLOSE), "<ctrl><shift>W");
    action_group.add_action_with_accel (new Gtk.ToggleAction ("document-split", _("Split Document"), _("Split the document view"), null), null);

    /* Help menu */
    action_group.add_action (new Gtk.Action ("help", _("_Help"), null, null));
    action_group.add_action (new Gtk.Action ("help-about", null, _("About this application"), Gtk.STOCK_ABOUT));

    /* Fullscreen toolbar */
    action_group.add_action_with_accel (new Gtk.Action ("leave-fullscreen", _("Leave fullscreen"), _("Leave the fullscreen mode"), Gtk.STOCK_LEAVE_FULLSCREEN), null);

    /* Project popup */
    action_group.add_action_with_accel (new Gtk.Action ("project-file-new", _("New file"), _("Create new source file"), Gtk.STOCK_NEW), null);
    action_group.add_action_with_accel (new Gtk.Action ("project-file-open", _("Open file"), _("Open source file"), Gtk.STOCK_OPEN), null);
    action_group.add_action_with_accel (new Gtk.Action ("project-file-add", _("Add file"), _("Add source file to project"), Gtk.STOCK_ADD), null);
    action_group.add_action_with_accel (new Gtk.Action ("project-file-remove", _("Remove file"), _("Remove this file from the project"), Gtk.STOCK_REMOVE), null);

    this.insert_action_group (this.action_group, 0);
  }

  /**
   * Get a Gtk.Action from name
   *
   * @param name The action name
   *
   * @return The Gtk.Action
   */
  public new Gtk.Action get_action (string name)
  {
    return this.action_group.get_action (name);
  }

  /**
   * Sets the checked state on the toggle action without signal emit
   *
   * @param name The action name
   * @param active Whether the action should be checked or not 
   */
  public void action_set_toggled (string name, bool active)
  {
    Gtk.ToggleAction action;

    action = this.get_action (name) as Gtk.ToggleAction;
    if (action != null)
    {
      this.action_group.remove_action (action);
      action.set_active (active);
      this.action_group.add_action (action);
    }
    else
    {
      debug ("Invalid action name: %s", name);
    }
  }

  private static const string xmldef="""<ui>		
  <menubar name="menubar">
    <menu action="file">
      <menuitem action="file-new"/>
      <placeholder name="file-ops-1"/>
      <menuitem action="file-open"/>
      <placeholder name="file-ops-2"/>
      <separator/>
      <menuitem action="file-save"/>
      <menuitem action="file-save-as"/>
      <placeholder name="file-ops-3"/>
      <separator/>
      <menuitem action="file-rename"/>
      <placeholder name="file-ops-4"/>
      <placeholder name="file-recents-placeholder">
        <separator/>
      </placeholder>
      <separator/>
      <menuitem action="file-close"/>
      <menuitem action="quit"/>
    </menu>

    <menu action="edit">
      <menuitem action="edit-undo"/>
      <menuitem action="edit-redo"/>
      <separator/>
      <menuitem action="edit-copy"/>
      <menuitem action="edit-cut"/>
      <menuitem action="edit-paste"/>
      <placeholder name="edit-ops-1"/>
      <separator/>
      <menuitem action="edit-lower"/>
      <menuitem action="edit-upper"/>
      <placeholder name="edit-ops-2"/>
      <separator/>
      <menuitem action="edit-preferences"/>
    </menu>

    <menu action="search">
      <menuitem action="search-find"/>
      <menuitem action="search-find-next"/>
      <menuitem action="search-find-prev"/>
      <placeholder name="search-ops-1"/>
      <separator/>
      <menuitem action="search-replace"/>
      <placeholder name="search-ops-2"/>
      <separator/>
      <menuitem action="search-goto-line"/>
      <placeholder name="search-ops-3"/>
      <separator/>
    </menu>

    <menu action="view">
      <menuitem action="view-toolbar"/>
      <menuitem action="view-statusbar"/>
      <separator/>
      <menuitem action="view-fullscreen"/>
      <placeholder name="view-layout-placeholder">
        <separator/>
      </placeholder>
    </menu>

    <menu action="project">
      <menuitem action="project-new"/>
      <menuitem action="project-open"/>
      <menuitem action="project-close"/>
      <placeholder name="project-ops-1"/>
      <separator/>
      <menuitem action="project-add" />
      <menuitem action="project-remove" />
      <placeholder name="project-ops-2"/>
      <separator/>
      <menuitem action="project-configure"/>
      <menuitem action="project-build"/>
      <menuitem action="project-install"/>
      <menuitem action="project-dist"/>
      <menuitem action="project-clean"/>
      <menuitem action="project-distclean"/>
      <menuitem action="project-uninstall"/>
      <placeholder name="project-ops-3"/>
      <separator/>
      <menuitem action="project-execute"/>
      <menuitem action="project-abort"/>
      <placeholder name="project-ops-4"/>
      <separator/>
      <menuitem action="project-options"/>
      <placeholder name="project-recents-placeholder">
        <separator/>
      </placeholder>
    </menu>

    <menu action="document">
      <menuitem action="document-save-all"/>
      <menuitem action="document-close-all"/>
      <separator/>
      <menuitem action="document-split"/>
      <placeholder name="document-ops-1">
        <separator/>
      </placeholder>
    </menu>

    <menu action="help">
      <menuitem action="help-about"/>
    </menu>
  </menubar>

  <toolbar name="toolbar">
    <toolitem action="file-new" />
    <toolitem action="file-open" />
    <toolitem action="file-save" />
    <separator/>
    <toolitem action="edit-undo" />
    <toolitem action="edit-redo" />
    <separator/>
    <toolitem action="search-find" />
    <toolitem action="search-replace" />
    <separator/>
    <toolitem action="project-build"/>
    <toolitem action="project-execute"/>
    <toolitem action="project-abort"/>
  </toolbar>

  <toolbar name="fullscreen-toolbar">
    <toolitem action="file-new" />
    <toolitem action="file-open" />
    <toolitem action="file-save" />
    <separator/>
    <toolitem action="edit-undo" />
    <toolitem action="edit-redo" />
    <separator/>
    <toolitem action="search-find" />
    <toolitem action="search-replace" />
    <separator/>
    <toolitem action="project-build"/>
    <toolitem action="project-execute"/>
    <toolitem action="project-abort"/>
    <separator expand="true"/>
    <toolitem action="leave-fullscreen"/>
  </toolbar>

  <popup name="project-popup">
    <menuitem action="project-file-new" />
    <menuitem action="project-file-open" />
    <menuitem action="project-file-add" />
    <menuitem action="project-file-remove" />
  </popup>

  <popup name="document-popup">
    <menuitem action="file-rename"/>
    <separator/>
    <menuitem action="file-save"/>
    <menuitem action="file-save-as"/>
    <separator/>
    <menuitem action="file-close"/>
  </popup>
</ui>""";
}

