public struct WizardInfo
{
  public string title;
  public string desc;
  public string folder;
  public string type;
  public string path;
}

public class WizardDialog : AbstractWizard
{
  /**
   * Validates the warings that might occur in the superclass.
   */
  private bool hasWarnings()
  {
    return false;
  }

  /**
   * Checks whether the baseclass is valid or not.
   */
  private bool validInterfaces(string interfaces, ref string status)
  {
    return true;
  }

  /**
   * Checks whether the baseclass is valid or not.
   */
  private bool validBase(string baseclass, ref string status)
  {
    return true;
  }

  /**
   * Checks whether the classname is valid or not.
   */
  private bool validName(string name,  ref string  status)
  {
    if (name.length >= 4)
    {
      Utils.setImageTo(this.widgets.class_valid,Gtk.STOCK_APPLY);
      return true;
    }
    else
    {    
      status = "Name to short...!";
      Utils.setImageTo(this.widgets.class_valid,Gtk.STOCK_CANCEL);
    }
    return false;
  }

  /**
   * Checks whether the path of the entry is valid or not.
   */
  private bool validPath(string path,  ref string  status)
  {
    var f = File.new_for_path(path);
    if (f.query_file_type(0, null) == FileType.DIRECTORY)
    {
      Utils.setImageTo(this.widgets.folder_valid,Gtk.STOCK_APPLY);
      return true;
    }
    else
    {
      status = "That is not a valid directory!";
      Utils.setImageTo(this.widgets.folder_valid,Gtk.STOCK_CANCEL);
    }
    return false;
  }

  private void validate ()
  {
    string classname = this.widgets.class_entry.get_text();  
    string path = this.widgets.folder_entry.get_text();
             
    var status_img = this.widgets.status_valid;
    string status = this.widgets.status.get_text();

    string baseclass = this.widgets.super_entry.get_text();
    string interfaces = this.widgets.interface_entry.get_text();
                 
    if (validName(classname, ref status)
        && validPath(path, ref status)
        && validBase(baseclass, ref status)
        && validInterfaces(interfaces, ref status))
    {
      //everything correct -> check warnings and show it
      if (hasWarnings())
      {
        Utils.setImageTo(status_img,Gtk.STOCK_DIALOG_WARNING);
      }
      else
      {
        Utils.setImageTo(status_img,Gtk.STOCK_APPLY);
        status = "OK";
      }
      this.widgets.create.sensitive = true;
    }
    else
    {         
      Utils.setImageTo(status_img,Gtk.STOCK_CANCEL);
      this.widgets.create.sensitive = false;
    }
    this.widgets.status.set_text(status);
  }

  public WizardDialog (WizardInfo info)
  {
    this.widgets.window.set_title (info.title);
    this.widgets.folder_entry.set_text (info.folder);
    this.widgets.folder_valid.set_from_stock (Gtk.STOCK_APPLY, Gtk.IconSize.MENU);
    this.widgets.class_entry.changed.connect (validate);
    this.widgets.folder_entry.changed.connect (validate);
    this.widgets.template_header.set_active (true);
    this.widgets.create.sensitive = false;

    switch (info.type)
    {
      case "class":
      break;
      case "interface":
        this.widgets.addmain.sensitive = false;
        this.widgets.base_table.sensitive = false;
      break;
      case "struct":
        this.widgets.addmain.sensitive = false;
      break;
      case "enum":
        this.widgets.addmain.sensitive = false;
        this.widgets.base_table.sensitive = false;
        this.widgets.interface_table.sensitive = false;
      break;
    }
  }

  /**
   * @see Gtk.Object.destroy
   */
  public void destroy ()
  {
    this.widgets.window.destroy();
    base.destroy();
  }
}

